// TestSafeApi.cpp: implementation of the CTestSafeApi class.
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SafeApiExample.h"
#include "TestSafeApi.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTestSafeApi::CTestSafeApi()
{

}

CTestSafeApi::~CTestSafeApi()
{

}

//////////////////////////////////////////////////////////////////////
// Load SafeAPI in memory
//////////////////////////////////////////////////////////////////////

void CTestSafeApi::loadSafeApi(IConvert convert, ICryptoHash cryptoHash, ICryptoSym safeapi)
{
    m_convert       = convert;
    m_cryptoHash    = cryptoHash;
	m_safeapi       = safeapi;
}

//////////////////////////////////////////////////////////////////////
// Test SafeAPI main APIs
//////////////////////////////////////////////////////////////////////

void CTestSafeApi::test()
{

	m_cryptoHash.createSeedFile("Seed File Generation", "Type on your keyboard to generate random data");
	AfxMessageBox("seed file done");
	
	m_cryptoHash.getRandomBytes ("12231234234212314234234234");
	AfxMessageBox("seed done");

    CString sVersion    =  m_safeapi.getVersion();

    COleVariant cOlevar     = m_convert.stringToBytes("This is a String to convert to byte array");
    CString sConvert        = m_convert.bytesToString(cOlevar);    
    CString sConvertHex     = m_convert.bytesToHexString(cOlevar);
    
    COleVariant cOlevar2    = m_convert.hexStringToBytes(sConvertHex);  
    CString sConvertFinal   = m_convert.bytesToString(cOlevar2);
    AfxMessageBox(sConvertFinal);

    CString sDisplay = "OK! \n";
    sDisplay        += "SafeAPI Java Version : ";
    sDisplay        += sConvert + " \nHexadecimal Format: " + sConvertHex;
    AfxMessageBox(sDisplay);
	

	CString sAlgorithm  = "Blowfish";
    CString sPassphrase = "notre passphrase";
    CString sEmail      = "test@safeapi.com";
    CString sError;

    //
    // We use VARIANT in COlevariant MFC Classes to pass or store binary arrays 
    //
	COleVariant coleSeed = "Tranform that string to binary seed";

    // Key Generation
    m_safeapi.genSecretKey(sAlgorithm, sPassphrase, 128,
                        coleSeed, sEmail, 1);

    CString sKeyId = sEmail + "_1_" + sAlgorithm;     
    AfxMessageBox("genSecretKey done! Key is stored using Key Id: " + sKeyId);

    // Error cheking
    if (! m_safeapi.isOperationOK())
    {
        sError = m_safeapi.getRawError();
        AfxMessageBox(sError);
    }

    //
    // Test hash of File
    //
    
    COleVariant varHash = m_cryptoHash.hashDataFile("MD5", "c:\\safelogic\\safeapi\\examples\\test\\abc.txt");
    
    CString sHash = m_convert.bytesToHexString(varHash);
    AfxMessageBox("Hash value of \"abc\": " + sHash);
      
    // Error cheking
    if (! m_cryptoHash.isOperationOK())
    {
        sError = m_cryptoHash.getRawError();
        AfxMessageBox(sError);
    }

    //
    // Buffer encryption/decryption tests
    // 

    // We set the string to encrypt to the COleVariant
    // Note:  we can pass also a byte array (unsigned char *)  as a CByteArray
    //        coleDec = byteArray

    CString sString = "Ceci est une phrase  chiffrer et  dechiffrer!";
    COleVariant coleDec = sString;
		
    // Encrypt the String
    COleVariant coleEnc  = m_safeapi.encryptBuffer(sKeyId, sPassphrase, coleDec);

    // Check Error
    if (m_safeapi.isOperationOK())
    {
        AfxMessageBox("Encryption successfull!");
    }
    else
    {
        sError = m_safeapi.getRawError();
        AfxMessageBox(sError);
        return;
    }

    // Decrypt the String
	COleVariant coleDec2 = m_safeapi.decryptBuffer(sKeyId, sPassphrase, coleEnc);

    // Check Error
    if (m_safeapi.isOperationOK())
    {
        AfxMessageBox("Decryption successfull!");
    }
    else
    {
        sError = m_safeapi.getRawError();
        AfxMessageBox(sError);
        return;
    }

    // Format the result String
    CString sStringDec = m_convert.bytesToString(coleDec2);

	if ( sString != sStringDec )
	{
		AfxMessageBox("Decryption failed! \n sStringDec :" + sStringDec + ":\n sString    :" + sString   + ":");
		return;
	}
	else
    {   

        CString sResult = "Decryption successfull! : " + sStringDec;
        AfxMessageBox(sResult);
    }

    // It's better to clean the Variant before leaving
    coleEnc.Clear();
    coleDec.Clear();

    AfxMessageBox("End!");

    return;

}
