// RawRSAExemple.java
//
// - SafeAPI key pair export
// - Encrypt & decrypt a buffer using an exported "raw" SafeAPI key pair

package com.safeapi.test;

import com.safeapi.CryptoAsymRawRSA;

public class RawRSAExemple
{	
	public static void main( String args[] )
		throws Exception
	{				
		// Get a RSA Key Pair reference as main & first argument
		// otherwise use npomereu@safelogic.com_1_RSA as key id
		
		String sKeyId = "npomereu@safelogic.com_1_RSA";
		
		if (args.length == 1)
		{
			sKeyId = args[0];
		}
		
		System.out.println("Using key id: " + sKeyId);
				
		// build SafeAPI instance
		CryptoAsymRawRSA safeapi = new CryptoAsymRawRSA() ;
	
		//
		// Step 1:
		// Export from the SafeAPI key a raw RSA public key 
		//
		
		// Load the public key
		safeapi.loadPublicKey(sKeyId);
		
		byte [] bN_Modulus  = safeapi.getModulus();
		byte [] bD_Exponent = safeapi.getPublicKeyExponent();
		
		//
		// Step 2:
		// Encrypt a buffer with a raw RSA public key, exported from SafeAPI
		//
		
		String sPlainText = "This is the plain text!";
		
		// Encrypt the buffer Using RAW key
		byte [] bEncrypted = safeapi.encryptBufferRawKey(bN_Modulus, 
													   bD_Exponent, 
													   sPlainText.getBytes());

		if(!safeapi.isOperationOK())
		{
			System.out.println("AsymEncryptBuffer Error: " + safeapi.getRawError()) ;
			return;
		}
				
			
		//
		// Step 3:
		// Export from the SafeAPI key a raw RSA private key
		//
		
		// Load the private key
		safeapi.loadPrivateKey(sKeyId, "nicolas".toCharArray());
		
		byte [] bD_Exponent2 = safeapi.getPrivateKeyExponent();
		byte [] bP_Factor	 = safeapi.getP();
		byte [] bQ_Factor	 = safeapi.getQ();		
		
		//
		// Step 4:
		// Decrypt a buffer with a rwa RSA private key, exported from SafeAPI
		//
	
		// // Decrypt back the buffer Using RAW key
		byte [] bDecrypted = safeapi.decryptBufferRawKey(bD_Exponent2, 
													   bP_Factor, 
													   bQ_Factor, 
													   bEncrypted);
		if(!safeapi.isOperationOK())
		{
			System.out.println("AsymDecryptBuffer: " + safeapi.getRawError()) ;
			return;
		}
		
		// This should print "This is the plain text!"
		System.out.println(new String(bDecrypted));
		
	}	
	
}
// End RawRSAExemple.java