//TestCryptBufferKey.java : SafeAPI Package.
//Class Description  : Auto TestCryptDir of  SafeAPI symmetric crypto
//					 
//22/11/99 15:30 GR - 1st Release
//15/07/00 19:55 NDP - New methods & throw fatal exception
//16/07/00 16:05 NDP - comments & new names
//16/09/00 18:15 NDP - new package name & throws new eception
//01/03/01 16:10 NDP - use GetSecretKey
//13/09/01 12:25 NDP - set back CSA_DEBUG to false afer 1.05b debug


package com.safeapi.test;
import java.util.Date;
import com.safeapi.CryptoSym;
import com.safelogic.utilx.Hex;

public class TestCryptBufferComp extends TestTimer
{
	// set to true to force debug
	public static boolean CSA_DEBUG = false;

	// Default Vector
	public static String DEFAULT_VECTOR = "FEDCBA9876543210";

	// attributes
	CryptoSym m_safeapi;

	private String m_sAlgo     		= "Blowfish";
	private String m_sEmail			= "TestCryptBuffer@safeapi.com";
	private String m_sPassphrase	= "notre passphrase";

	private String m_sHexaKey = null;
	
	// Key ID built from e mail
	String m_sKeyId;

	public TestCryptBufferComp()
	{
	    m_safeapi = new CryptoSym();
	}

	/**
	 * M A I N
	 */
	public static void main(String[] args)
		throws Exception 
	{    
	    TestCryptBufferComp test = new TestCryptBufferComp();
	   
	    String sIn = "Bonjour  Tous!";
	    if (args.length == 1)
	    {
	        sIn = args[0];
	    }
	    
	    test.testGenKey();
	    
	    //test.cryptDecryptBufferKey(sIn);
	    
	    //Ok, loop
	    
	    int nEnd = 1000;
	    
        System.out.println("");
	    System.out.println("Test with passed key");
        System.out.println(new Date());
	    
	    for (int i = 0; i < nEnd; i++)
	    {
	        test.cryptDecryptBufferKey(sIn);
	    }
	    
	    System.out.println(new Date());
	    
	    System.out.println("");
	    
	    System.out.println("Test with key in File");
        System.out.println(new Date());
	    
	    for (int i = 0; i < nEnd; i++)
	    {
	        test.cryptDecryptBufferKey(sIn);
	    }	    

        System.out.println(new Date());
        
        
	}
	
	// methods

	/**
	 * generate a secret key
	 */
	public void testGenKey()
	{
		int nIndex = 1;

		byte[] bSeed = new byte[16];

		byte[] bSeed192 = m_safeapi.getRandomBytes(this.m_sPassphrase);
		DebugPrintln("bSeed192.length :" + bSeed192.length);

		System.out.println();
		System.out.println("*** Testing Symmetric Key Generation ***");
		System.out.println();

		// Strong seedind
		for (int nCnt = 0; nCnt < bSeed.length; nCnt++)
		{
			//DebugPrintln("nCnt :" + nCnt);
			bSeed[nCnt] = bSeed192[nCnt];
		}

		beginOp("Generating Key for algorithm " + m_sAlgo);

		this.m_safeapi.genSecretKey(
			m_sAlgo,
			m_sPassphrase.toCharArray(),
			128,
			bSeed,
			m_sEmail,
			nIndex);

		if (m_safeapi.isOperationOK())
		{
			m_sKeyId = m_sEmail + "_" + nIndex + "_" + m_sAlgo;
			stopOp("Key " + m_sKeyId + " generated");
		}
		else
		{
			System.out.println("Error : " + m_safeapi.getRawError());
		}
		
		String sHexaKey =
			m_safeapi.getSecretKey(
				this.m_sAlgo,
				this.m_sPassphrase.toCharArray(),
				this.m_sEmail,
				1);
		
		System.out.println("sHexaKey :" + sHexaKey + ":");
		
		m_sHexaKey = sHexaKey;
		
	}


	
	/**
	 * encrypt the specified String
	 * 
	 * @param	sDec String to encrypt
	 * @return	the  resulting encrypted String
	 */

	public String symEncryptKey(String sDec) throws Exception
	{
		
		byte [] bEnc =
			m_safeapi.encryptBufferKey(this.m_sAlgo,
			        					m_sHexaKey.toCharArray(), 
			        					sDec.getBytes());

		if (! m_safeapi.isOperationOK())
		{
			System.out.println("Error : 1"); // + m_safeapi.getRawError()) ;
			throw new Exception("Failed: " + sDec + " encryption/decryption");
		}

		return Hex.toString(bEnc);
	}

	/**
	 * decrypt the specified String
	 * 
	 * @param	sEnc String to encrypt
	 * @return	the  resulting encrypted String
	 */

	public String symDecryptKey(String sEnc) throws Exception
	{
	    
	   byte []bEnc = Hex.fromString(sEnc);
	   
		byte[] bDec =
			m_safeapi.decryptBufferKey(this.m_sAlgo,
					m_sHexaKey.toCharArray(), 
					bEnc);

		if (! m_safeapi.isOperationOK())
		{
			System.out.println("Error : 1"); // + m_safeapi.getRawError()) ;
			throw new Exception("Failed: " + sEnc + " encryption/decryption");
		}
		
		String sDec = new String(bDec);		
		return sDec;
	}

	/**
	 * Test symmetric methods: buffer encryption/decryption
	 * @param	sDec	Buffer to use for encryption/decryption
	 */

	public void cryptDecryptBufferKey(String sDec) throws Exception
	{
		String sEnc = new String();
		String sDec2 = new String();

		sEnc = this.symEncryptKey(sDec);
		sDec2 = this.symDecryptKey(sEnc);

		this.compString(sDec, sDec2);
	}


    /**
     * comparaison of two Strings
     * @param s1    String to compare
     * @param s2    String  compare with s2
     */

    public void compString(String s1, String s2) throws Exception
    {
        if (s1.equals(s2))
        {
            //System.out.println("comp: " + s1 + "=" + s2);
        }
        else
        {
            throw new Exception("comp: " + s1 + "!=" + s2);
        }
    }
    
	// safeLogic debug tools

	public static void SetDebug(boolean bDebug)
	{
		CSA_DEBUG = bDebug;
	}

	private static void DebugPrintln(String sIn)
	{
		if (CSA_DEBUG)
			System.out.println(sIn);
	}

}

//End
