// TestHashBase.java : SafeAPI Package.
// Class Description       : Auto TestCryptDir of MD5 & SHA-1
//						 
//
// 16/07/00 19:35 NDP- Creation
// 16/09/00 17:50 NDP - works with dynamic files & with unix
// 16/09/00 17:50 NDP - new package name
// 09/01/03 19:45 NDP - use utilx only
// 09/01/03 19:45 NDP - use utilx only

package com.safeapi.test;

import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;
import com.safeapi.Convert;
import com.safeapi.CryptoHash;
import com.safelogic.utilx.Hex;
import com.safelogic.utilx.dir.Directory;

public class TestHashBase
{
	public static String TEST_FILE = "safeapi_hash_test.txt";
	
	// Vector of Hash input/hash output
	Vector m_vHashTestList ;
	
	/**
	 * Constructor
	 */
	public TestHashBase()
	{	
		createList();
	}
	
	/**
	 * Create lists of input/output ref values
	 */
	public void createList()
	{
		m_vHashTestList = new Vector() ;
		
		m_vHashTestList.addElement(new TestHashRef("MD5",
														 "a",
														 "0cc175b9c0f1b6a831c399e269772661")) ;
		
		m_vHashTestList.addElement(new TestHashRef("MD5",
														 "abc",
														 "900150983cd24fb0d6963f7d28e17f72")) ;
		
		
		m_vHashTestList.addElement(new TestHashRef("MD5",
														 "message digest",
														 "f96b697d7cb7938d525a2f31aaf161d0")) ;
		
		
		m_vHashTestList.addElement(new TestHashRef("MD5",
														 "abcdefghijklmnopqrstuvwxyz",
														 "c3fcd3d76192e4007dfb496cca67e13b")) ;
		
		
		m_vHashTestList.addElement(new TestHashRef("MD5",
														 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
														 "d174ab98d277d9f5a5611c2c9f419d9f")) ;
		
		
		m_vHashTestList.addElement(new TestHashRef("MD5",
														 "12345678901234567890123456789012345678901234567890123456789012345678901234567890",
														 "57edf4a22be3c955ac49da2e2107b67a")) ;
		
		
		m_vHashTestList.addElement(new TestHashRef("SHA-1",
														 "abc",
														 "A9993E364706816ABA3E25717850C26C9CD0D89D")) ;
		
		
		m_vHashTestList.addElement(new TestHashRef("SHA-1",
														 "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
														 "84983E441C3BD26EBAAE4AA1F95129E5E54670F1")) ;

	}

	/**
	 * compute the reference TestCryptDir for hash algorithms of SafeAPI : buffer mode
	 */
	
	public void testHashBuffers()
		throws Exception
	{
		TestHashRef hrCurrent ;
		
		System.out.println("          #####################################") ;
		System.out.println("          #   Hash Algorithms  : TestCryptDir buffers #") ;
		System.out.println("          #####################################") ;	
				
		for(int nCnt = 0 ; nCnt < m_vHashTestList.size() ; nCnt ++)
		{
			hrCurrent = (TestHashRef) m_vHashTestList.elementAt(nCnt) ;
			
			System.out.println("***********************************************");
			System.out.println("Test Reference for " + hrCurrent.getAlgo()) ;
			System.out.println("***********************************************");
			System.out.println() ;
			
			System.out.println("Input           : ") ;
			System.out.print(hrCurrent.getRefIn()) ;
			System.out.println() ;
			
			System.out.println("Output Reference: ") ;
			System.out.print(hrCurrent.getRefOut().toLowerCase()) ;
			System.out.println() ;
			
			System.out.println("Output SafeAPI  : ") ;
			String sCompute = getHexaHash(hrCurrent.getAlgo(), hrCurrent.getRefIn());
			System.out.print(sCompute);
			System.out.println() ;
			
			if ( !hrCurrent.getRefOut().equalsIgnoreCase(sCompute))
			{
				throw new Exception(" ***FAILED " + hrCurrent.getAlgo());
			}
		}
		
		System.out.println("***********************************************") ;
	}
	
	
	/**
	 * compute the reference TestCryptDir for hash algorithms of SafeAPI : buffer mode
	 */
	
	public void testHashFiles()
		throws Exception
	{
		TestHashRef hrCurrent ;
		
		System.out.println("          #####################################") ;
		System.out.println("          #   Hash Algorithms  : TestCryptDir files   #") ;
		System.out.println("          #####################################") ;	
				
		for(int nCnt = 0 ; nCnt < m_vHashTestList.size() ; nCnt ++)
		{
			hrCurrent = (TestHashRef) m_vHashTestList.elementAt(nCnt) ;
			
			System.out.println("***********************************************");
			System.out.println("Test Reference for " + hrCurrent.getAlgo()) ;
			System.out.println("***********************************************");
			System.out.println() ;
			
			System.out.println("Input           : ") ;
			System.out.print(hrCurrent.getRefIn()) ;
			System.out.println() ;
			
			System.out.println("Output Reference: ") ;
			System.out.print(hrCurrent.getRefOut().toLowerCase()) ;
			System.out.println() ;
			
			System.out.println("Output SafeAPI  : ") ;
			
			// Create the file
			createFileFromString(hrCurrent.getRefIn());
			
			CryptoHash safeapi	= new CryptoHash();
			
			 byte [] bCompute= 
				safeapi.hashDataFile(hrCurrent.getAlgo(), TEST_FILE);
			 
			String sCompute = Hex.toString(bCompute);
			System.out.print(sCompute);
			System.out.println() ;
			
			if ( !hrCurrent.getRefOut().equalsIgnoreCase(sCompute))
			{
				throw new Exception(" ***FAILED " + hrCurrent.getAlgo());
			}
		}
		
		System.out.println("***********************************************") ;
	}
	
	
	/**
	 * Create a TestCryptDir file with a content
	 * @param	sContent	File content
	 */
	
	private void createFileFromString(String sContent)
		throws IOException
	{		
		String sFile = new String();
		sFile = System.getProperty("user.dir");
		sFile = Directory.AddSep(sFile);
		sFile+= TEST_FILE;
		
		DataOutputStream dos = new DataOutputStream(new FileOutputStream(sFile));
		dos.writeBytes(sContent);
		dos.close();
	}
	
	/**
	 * Compute a hash value in hex format
	 * @param	sAlgo		MD5 or SHA-1
	 * @param	sIn		String to hash
	 * @return	digest hex string
	 */
	
	public String getHexaHash(String sAlgo, String sIn)
	{
		Convert ccvConverter = new Convert() ;	
		CryptoHash safeapi = new CryptoHash();
		
		byte[] bDigest ;
		safeapi.hashDataBufferInit(sAlgo) ;
		safeapi.hashDataBuffer(sIn.getBytes()) ;
		bDigest = safeapi.hashDataBufferDigest() ;
		
		if (! safeapi.isOperationOK())
		{
		    System.out.println(safeapi.getRawError());
		    System.out.println(safeapi.getRegisteredError());
		    System.exit(0);
		}
		
		return ccvConverter.bytesToHexString(bDigest);
	
	}
	
				
}	
// END