// TestRSA.java : SafeAPI Package.
// Class Description  : Auto TestCryptDir of  SafeAPI asymmetric crypto
//					 
// 13/06/02 18:45 NDP - creation

package com.safeapi.test;

import com.safeapi.CryptoAsymRawRSA;
import com.safeapi.util.ParmsInternal;
import com.safelogic.crytools.ParmCry;
import com.safelogic.utilx.Hex;

/**
 * This Class is designed to TestCryptDir the rsa() methods of SafeAPI
 */

public class TestRSA extends TestTimer
{
	// set to true to force debug
	public static boolean CSA_DEBUG = false;
	
	public static String STRING1 
		=  "Longtemp, je me suis couch de bonne heure. Parfois,  peine ma bougie teinte, mes yeux se fermaient si vite que je navais pas le temps de me dire : Je mendors. Et, une demi-heure aprs, la pense quil tait temps de chercher le sommeil mveillait ; j";
	
	public static String STRING2
		=  "e voulais poser le volume que je croyais avoir encore dans les mains et souffler ma lumire ; je navais pas cess en dormant de faire des rflexions sur ce que je venais de lire, mais ces rflexions avaient pris un tour un peu particulier ; il me semblait";		   

	public static String STRING3
		= " que jtais moi-mme ce dont parlait louvrage : une glise, un quatuor, la rivalit de Franois Ier et de Charles Quint. Cette croyance survivait pendant quelques secondes  mon rveil **fin**";
	
	// attributes
	CryptoAsymRawRSA m_ccaSafeTest ;
	
	String m_sEmail;
	String m_sPassphrase ;	
	
	
	/**
	 * Constructor
	 * 
	 * @param sEmail		E mail used to builds the symmetric key
	 * @param sPasshrase	passphrase for key
	 */
	
	public TestRSA(String sEmail,
							  String sPasshrase)
	{
		// Create a SafeAPI instance
		super() ;
		m_ccaSafeTest = new CryptoAsymRawRSA() ;
		
		this.m_sEmail		= sEmail;
		this.m_sPassphrase	= sPasshrase;
	}

	/**
	 * M A I N
	 */
	public static void main(String[] args)
		throws Exception 
	{
		//String sText = "Hello, World!";
		
		String sText =	STRING1 + STRING2 + STRING3;
		
		//System.out.println(":" + sText + ":");
		
		TestRSA rsaCrypt = new TestRSA(all.KEY_EMAIL, all.KEY_PASS);
		
		String sEnc = rsaCrypt.testRSAEncrypt(sText);		
		String sDec = rsaCrypt.testRSADecrypt(sEnc);
			
		//System.out.println(":" + sDec + ":");
		
		rsaCrypt.testCompString(sText, sDec);
	}
	
	
	/**
	 * RSA encrypt the specified String with a RAW RSA key
	 * 
	 * @param	sDec String to encrypt
	 * @return	the  resulting encrypted String 
	 */
	
	public String testRSAEncrypt(String sDec)
		throws Exception 
	{		
		System.out.println() ;
		System.out.println("*** Testing RSA Algorithm - 1) encrypt (1.06c) ***") ;
		System.out.println() ;
		
		int nCnt = 1;
		String sKeyId = new String() ;
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;		
		
		m_ccaSafeTest.loadPublicKey(sKeyId);
		
		byte [] bN_Modulus  = m_ccaSafeTest.getModulus();
		byte [] bD_Exponent = m_ccaSafeTest.getPublicKeyExponent();
		
		// Using RSA key elements
		byte [] bEnc = m_ccaSafeTest.rsa(bN_Modulus, 
										 bD_Exponent, 
										 sDec.getBytes());

		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println("rsa: " + m_ccaSafeTest.getRawError()) ;
			return null;
		}
		
		return new String(Hex.toString(bEnc));

	}	
	
	
	/**
	 * RSA decrypt the specified String
	 * 
	 * @param	sEnc String to decrypt
	 * @return	the  resulting encrypted String 
	 */
	
	public String testRSADecrypt(String sEnc)
		throws Exception 
	{		
		System.out.println() ;
		System.out.println("*** Testing RSA Algorithm - 2) decrypt (1.06c) ***") ;
		System.out.println() ;
		
		int nCnt = 1;
		String sKeyId = new String() ;
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;		
			
		m_ccaSafeTest.loadPrivateKey(sKeyId, m_sPassphrase.toCharArray());
		
		byte [] bModulus			= m_ccaSafeTest.getModulus();
		byte [] bD_PrivateExponent	= m_ccaSafeTest.getPrivateKeyExponent();
		byte [] bP_Factor			= m_ccaSafeTest.getP();
		byte [] bQ_Factor			= m_ccaSafeTest.getQ();
		byte [] bInverse			= m_ccaSafeTest.getInverseOfQModP();			
		
		// Using RSA key elements
		byte [] bDec = m_ccaSafeTest.rsaWithCrt(bModulus,
												bD_PrivateExponent, 
												bP_Factor, 
												bQ_Factor, 
												bInverse,
												Hex.fromString(sEnc));
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println("AsymDecryptBuffer: " + m_ccaSafeTest.getRawError()) ;
			return null;
		}
		
		return new String(bDec);
	}	

	
	/**
	 * Test symmetric methods: buffer encryption/decryption
	 * @param	sDec	Buffer to use for encryption/decryption
	 */
	
	public void testRSACryptDecrypt(String sDec)
		throws Exception
	{				
		String sEnc = new String();  
		String sDec2= new String();  
		
		sEnc  = this.testRSAEncrypt(sDec);
		sDec2 = this.testRSADecrypt(sEnc);
		
		this.testCompString(sDec, sDec2);	
	}	
		
	/**
	 * comparaison of two Strings
	 * @param s1	String to compare
	 * @param s2	String  compare with s2
	 */
	
	public void testCompString(String s1, String s2)
		throws Exception
	{
		if (s1.equals(s2))
		{
			System.out.println("comp: " + s1 + "=" + s2);
		}
		else {
			throw new Exception("comp: " + s1 + "!=" + s2);
		}
	}		
	
	// safeLogic debug tools
	
	public static void SetDebug(boolean bDebug)
	{
		CSA_DEBUG = bDebug;
	}	
	
	private static void DebugPrintln(String sIn)
	{
		if (CSA_DEBUG)
			System.out.println(sIn);
	}		
}
	
