// TestRSACrypt.java : SafeAPI Package.
// Class Description  : Auto TestCryptDir of  SafeAPI asymmetric crypto
//					 
// 19/11/99 00:50 GR - Release
// 20/11/99 01:25 GR - Add comments
// 24/02/00 12:30 GR - testKeyPairGen() -> testKeyPairGen(int nKeySize)
// 16/07/00 19:15 NDP- adapt to use with CSaCryAutoTest & comments
// 17/07/00 21:10 NDP- Double RSA Key Generation
// 16/09/00 17:50 NDP - new package name
// 06/10/00 12:35 NDP - use safeAPI v1.04 algorithm
// 01/03/01 13:55 NDP - use SafeAPI v1.05 new raw RSA methods
// 01/03/01 20:00 NDP - add-back package option
// 05/06/01 14:30 NDP - comments
// 04/06/02 19:45 NDP - use new raw RSA key exports methods
// 06/06/02 18:45 NDP - clean comments
// 18/04/03 19:30 NDP - No Recipients List where decrypting file with rsa

package com.safeapi.test;

import com.safeapi.CryptoAsym;
import com.safeapi.CryptoAsymRawRSA;
import com.safeapi.CryptoHash;
import com.safeapi.util.ParmsInternal;
import com.safelogic.crytools.ParmCry;
import com.safelogic.utilx.Hex;



/**
 * This Class is designed to TestCryptDir the public key interface of SafeAPI.
 * <br>
 * This includes all asymmetric encryption and signature methods.
 * 
 */

public class TestRSACrypt extends TestTimer
{
	// set to true to force debug
	public static boolean CSA_DEBUG = false;
	
	// attributes
    CryptoHash m_hash;
    CryptoAsym  m_ccaSafeAsym ;
    CryptoAsymRawRSA m_ccaSafeTest ;
	
	String m_sEmail;
	String m_sPassphrase ;	
	
	
	/**
	 * Constructor
	 * 
	 * @param sEmail		E mail used to builds the symmetric key
	 * @param sPasshrase	passphrase for key
	 */
	
	public TestRSACrypt(String sEmail,
							  String sPasshrase)
	{
		// Create a SafeAPI instance
		super() ;
		m_ccaSafeTest = new CryptoAsymRawRSA() ;
		m_hash = new CryptoHash();
        m_ccaSafeAsym = new CryptoAsym();
		this.m_sEmail		= sEmail;
		this.m_sPassphrase	= sPasshrase;
	}

	/**
	 * M A I N
	 */
	public static void main(String[] args)
		throws Exception 
	{
		String sText = "Hello, World!";
		System.out.println(":" + sText + ":");
		
		TestRSACrypt rsaCrypt = new TestRSACrypt(all.KEY_EMAIL, all.KEY_PASS);
		
		String sEnc = rsaCrypt.testRSAEncryptBuffer(sText);		
		String sDec = rsaCrypt.testRawRSADecryptBuffer(sEnc);
		
		System.out.println(":" + sDec + ":");
	}
	
	
	/**
	 * RSA encrypt the 128 bits specified String with a RAW RSA key
	 * 
	 * @param	sDec String to encrypt
	 * @return	the  resulting encrypted String 
	 */
	
	public String testRawRSAEncryptBuffer(String sDec)
		throws Exception 
	{		
		System.out.println() ;
		System.out.println("*** Testing Raw RSA Buffer Encryption (1.06b) ***") ;
		System.out.println() ;
		
		int nCnt = 1;
		String sKeyId = new String() ;
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;		
		
		m_ccaSafeTest.loadPublicKey(sKeyId);
		
		byte [] bN_Modulus  = m_ccaSafeTest.getModulus();
		byte [] bD_Exponent = m_ccaSafeTest.getPublicKeyExponent();
		
		// Using RAW key
		byte [] bEnc = m_ccaSafeTest.encryptBufferRawKey(bN_Modulus, bD_Exponent, sDec.getBytes());

		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println("AsymEncryptBuffer: " + m_ccaSafeTest.getRawError()) ;
			return null;
		}
		
		return new String(Hex.toString(bEnc));

	}	
	
	
	/**
	 * RSA decrypt the 128 bits specified String
	 * 
	 * @param	sEnc String to decrypt
	 * @return	the  resulting encrypted String 
	 */
	
	public String testRawRSADecryptBuffer(String sEnc)
		throws Exception 
	{		
		System.out.println() ;
		System.out.println("*** Testing Raw RSA Buffer Decryption (1.06b) ***") ;
		System.out.println() ;
		
		int nCnt = 1;
		String sKeyId = new String() ;
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;		
			
		m_ccaSafeTest.loadPrivateKey(sKeyId, m_sPassphrase.toCharArray());
		
		byte [] bD_Exponent2 = m_ccaSafeTest.getPrivateKeyExponent();
		byte [] bP_Factor	 = m_ccaSafeTest.getP();
		byte [] bQ_Factor	 = m_ccaSafeTest.getQ();		
		
		// Using RAW Key
		byte [] bDec = m_ccaSafeTest.decryptBufferRawKey(bD_Exponent2, bP_Factor, bQ_Factor, 
													   Hex.fromString(sEnc));
		
		// Using SafeAPI internal keys
		//byte [] bDec = m_safeapi.AsymDecryptBuffer(sKeyId,
		//											   this.m_sPassphrase, Hex.fromString(sEnc));
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println("AsymDecryptBuffer: " + m_ccaSafeTest.getRawError()) ;
			return null;
		}
		
		return new String(bDec);
	}	

	
	/**
	 * Test symmetric methods: buffer encryption/decryption
	 * @param	sDec	Buffer to use for encryption/decryption
	 */
	
	public void testRawRSACryptDecryptBuffer(String sDec)
		throws Exception
	{				
		String sEnc = new String();  
		String sDec2= new String();  
		
		sEnc  = this.testRawRSAEncryptBuffer(sDec);
		sDec2 = this.testRawRSADecryptBuffer(sEnc);
		
		this.testCompString(sDec, sDec2);	
	}	
		
	
	
	// methods previous to v 1.05a
	
	/**
	 * generate two key pairs (necessary to use the other TestCryptDir methods).
	 * <BR>
	 * You have to use this at least the first time you launch this TestCryptDir package.
	 */
	
	public void testKeyPairGen(int nKeySize)
	{		
		byte[] bSeed192 = m_ccaSafeTest.getRandomBytes(this.m_sPassphrase);
		
        DebugPrintln("bSeed192.length :" + bSeed192.length);
		
		byte[] bSeed = new byte[16] ;
		
		System.out.println() ;
		System.out.println("*** Testing RSA Key Generation ***") ;
		System.out.println() ;
		
		// Strong seedind
		for(int nCnt = 0 ; nCnt < bSeed.length ; nCnt++)
		{
			//DebugPrintln("nCnt :" + nCnt);
			bSeed[nCnt] = bSeed192[nCnt];
		}
		
		for(int nKeyIndex = 1 ; nKeyIndex < 3 ; nKeyIndex++)
		{
			
			beginOp("Generating KeyPair " 
					+ this.m_sEmail 
					+ "_" + nKeyIndex + "_" + "RSA ("+ nKeySize + " bits)") ;
			
			m_ccaSafeAsym.genKeyPair(ParmCry.CSA_ALGO_RSA,
									 this.m_sPassphrase.toCharArray(),
									 nKeySize,
									 bSeed,
									 this.m_sEmail,
									 nKeyIndex) ;
		}
		
		if(!m_ccaSafeAsym.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
		
		stopOp("Key generated") ;		
	}	
	
	
	/**
	 * RSA encrypt the 128 bits specified String
	 * 
	 * @param	sDec String to encrypt
	 * @return	the  resulting encrypted String 
	 */
	
	public String testRSAEncryptBuffer(String sDec)
		throws Exception 
	{		
		System.out.println() ;
		System.out.println("*** Testing RSA Buffer Encryption ***") ;
		System.out.println() ;
		
		int nCnt = 1;
		String sKeyId = new String() ;
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;		
		
		byte [] bEnc = m_ccaSafeAsym.encryptBuffer(sKeyId,
													   sDec.getBytes());
		
		if(!m_ccaSafeAsym.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return null;
		}
		
		return new String(Hex.toString(bEnc));
	}
	
	/**
	 *  RSA decrypt the 128 bits specified String
	 * 
	 * @param	sEnc String to encrypt
	 * @return	the  resulting encrypted String
	 */
	
	
	public String testRSADecryptBuffer(String sEnc)
		throws Exception 
	{
		System.out.println() ;
		System.out.println("*** Testing RSA Buffer Decryption ***") ;
		System.out.println() ;
		
		int nCnt = 1;
		String sKeyId = new String() ;
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;		
		
		byte [] bDec = m_ccaSafeAsym.decryptBuffer(sKeyId,
													   this.m_sPassphrase.toCharArray(),
													   Hex.fromString(sEnc));
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return null;
		}
		
		return new String(bDec);
	}	
	
	/**
	 * Test symmetric methods: buffer encryption/decryption
	 * @param	sDec	Buffer to use for encryption/decryption
	 */
	
	public void testRSACryptDecryptBuffer(String sDec)
		throws Exception
	{				
		String sEnc = new String();  
		String sDec2= new String();  
		
		sEnc  = this.testRSAEncryptBuffer(sDec);
		sDec2 = this.testRSADecryptBuffer(sEnc);
		
		this.testCompString(sDec, sDec2);	
	}	
	
	/**
	 * comparaison of two Strings
	 * @param s1	String to compare
	 * @param s2	String  compare with s2
	 */
	
	public void testCompString(String s1, String s2)
		throws Exception
	{
		if (s1.equals(s2))
		{
			System.out.println("comp: " + s1 + "=" + s2);
		}
		else {
			throw new Exception("comp: " + s1 + "!=" + s2);
		}
	}		
	
	/**
	 * the TestCryptDir method for the asymmetric encryption mecanisms of SafeAPI
	 * 
	 * @param	sFileDec	the file to encrypt
	 * @param	sFileEnc	the  resulting encrypted file
	 */
	
	public void testRSAEncryptFile(String sFileDec , String sFileEnc)
		throws Exception 
	{
		System.out.println() ;
		System.out.println("*** Testing RSA File Encryption ***") ;
		System.out.println() ;
		
		String sKeyId = new String() ;
		int nCnt = 1;
		
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;		
		
		// Create & add a recipient list
		m_ccaSafeAsym.createRecipients("Recipient_List") ;
		m_ccaSafeAsym.addRecipient("Recipient_List", sKeyId) ;	
		
		if(!m_ccaSafeAsym.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
		
		// Crypt a TestCryptDir file to recipients with each public key
		
		beginOp("RSA Encrypting file " + sFileEnc) ;
		
		m_ccaSafeAsym.encryptFile("Recipient_List",                                       
                                      sFileDec,
									  sFileEnc
									  );
		
		System.out.println("RSA Encrypting file : terminated " + sFileEnc) ;
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
		
		return;
	}
	
		
	/**
	 * the TestCryptDir method for the asymmetric encryption mecanisms of SafeAPI
	 * 
	 * @param	sFileEnc	the encrypted file 
	 * @param	sFileDec	the resulting decrypted file
	 */
	
	public void testRSADecryptFile(String sFileEnc, String  sFileDec)
		throws Exception 
	{
		System.out.println() ;
		System.out.println("*** Testing RSA File Decryption ***") ;
		System.out.println() ;
		
		String sKeyId = new String() ;
		int nCnt = 1;
			
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;		
		
		// Create & add a recipient list
		//System.out.println("*** (No Recipients List) ***") ;
		//m_safeapi.CreateRecipients("Recipient_List") ;
		//m_safeapi.AddRecipient("Recipient_List", sKeyId) ;	
		
		//if(!m_safeapi.isOperationOK())
		//{
		//	System.out.println(m_safeapi.getRawError()) ;
		//	return ;
		//}
		
		beginOp("Decrypting file " + sFileEnc) ;
		
		m_ccaSafeAsym.decryptFile(sKeyId,
									  this.m_sPassphrase.toCharArray(),
									  sFileEnc,
									  sFileDec) ;
		
		stopOp(" decrypted file : " + sFileDec) ;
		
		if(!m_ccaSafeAsym.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
			
	}
	
	/**
	 * Test symmetric methods: File encryption/decryption
	 * @param	sFile	File to use for encryption/decryption
	 */
	
	public void testRSACryptDecryptFile(String sFileDec)
		throws Exception
	{				
		String sFileEnc = sFileDec + ".rsa_enc";
		String sFileDec2= sFileDec + ".rsa_dec";
		
        //new File(sFileEnc).delete();
		//new File(sFileDec2).delete();
		
		this.testRSAEncryptFile(sFileDec, sFileEnc);
		this.testRSADecryptFile(sFileEnc, sFileDec2);
		this.testCompFile(sFileDec, sFileDec2);
		
	}	
	
	/**
	 * MD5 compare of two files
	 * @param sFile1	File to compare
	 * @param sFile2	File to MD5 compare with File1
	 */
	
	public void testCompFile(String sFile1 , String sFile2)
		throws Exception
	{		
		String sHash1 = Hex.toString( this.m_hash.hashDataFile("MD5", sFile1));
		String sHash2 = Hex.toString( this.m_hash.hashDataFile("MD5", sFile2));
		
		if (sHash1.equals(sHash2))
		{
			System.out.println("MD5 comp: " + sFile1 + "=" + sFile2);
		}
		else {
            //throw new Exception();
			throw new Exception("MD5 comp: " + sFile1 + "!=" + sFile2);
		}
	}	
	
	// safeLogic debug tools
	
	public static void SetDebug(boolean bDebug)
	{
		CSA_DEBUG = bDebug;
	}	
	
	private static void DebugPrintln(String sIn)
	{
		if (CSA_DEBUG)
			System.out.println(sIn);
	}		
}
	
