// TestRSACryptDir.java : SafeAPI Package.
// Class Description  : Auto TestCryptDir of  SafeAPI asymmetric crypto
//					 
// 28/04/03 19:30 NDP - Creation

package com.safeapi.test;

import com.safeapi.CryptoDir;
import com.safeapi.util.ParmsInternal;
import com.safelogic.crytools.ParmCry;
import com.safelogic.crytools.SafeProvider;

/*
 * This Class is designed to TestCryptDir the public key interface of SafeAPI.
 * <br>
 * This includes all asymmetric encryption for directories
 * 
 */

public class TestRSACryptDir extends TestTimer
{
	
    // set to true to force debug
    public static boolean CSA_DEBUG = false;
    
	public static final int ENCRYPT_MODE = 1;
	public static final int DECRYPT_MODE = 2;    	
		
    String m_sEmail;
    String m_sPassphrase ;  
    
    CryptoDir  m_ccaSafeAsym ;
    
	/**
	 * Constructor
	 * 
	 * @param sEmail		E mail used to builds the symmetric key
	 * @param sPasshrase	passphrase for key
	 */
	
	public TestRSACryptDir(String sEmail,
						   String sPasshrase)
	{
		// Create a SafeAPI instance
        m_sEmail = sEmail;
        m_sPassphrase = sPasshrase;
        
        m_ccaSafeAsym = new CryptoDir();
	}

	/**
	 * M A I N
	 */
	public static void main(String[] args)
		throws Exception 
	{		
		if (args.length == 0 || args.length != 3)
		{
			System.out.println("TestRSACryptDir inputDir OutputDir <1 (encrypt) | 2 (decrypt)> ");
			return;
		}		
		// Add Cryptix provider dynamically
		
        SafeProvider.add();      		
		
		String sInputDir = null;
		String sOutputDir = null;
		int nMode = 0;
        
		sInputDir  = args[0];        
		sOutputDir = args[1];
		nMode = new Integer(args[2]).intValue();
		
		TestRSACryptDir rsaCryptDir 
			= new TestRSACryptDir(all.KEY_EMAIL, all.KEY_PASS);	
		
		if (nMode == ENCRYPT_MODE )
		{
			rsaCryptDir.testRSAEncryptDir(sInputDir, sOutputDir);
		}
		else if (nMode == DECRYPT_MODE )
		{
			rsaCryptDir.testRSADecryptDir(sInputDir, sOutputDir);
		}		
									
	}
	
	/**
	 * the TestCryptDir method for the asymmetric encryption mecanisms of SafeAPI
	 * 
	 * @param	sInputDir	the directory to encrypt
	 * @param	sOutputDir	the resulting encrypted directory
	 */
	
	public void testRSAEncryptDir(String sInputDir, String sOutputDir)
		throws Exception 
	{
		System.out.println() ;
		System.out.println("*** Testing RSA Directory Encryption ***") ;
		System.out.println() ;
		
		String sKeyId = new String() ;
		int nCnt = 1;
		
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;		
		
		// Create & add a recipient list
        m_ccaSafeAsym.createRecipients("Recipient_List") ;
        m_ccaSafeAsym.addRecipient("Recipient_List", sKeyId) ;	

		if(!m_ccaSafeAsym.isOperationOK())
		{
			System.out.println(m_ccaSafeAsym.getRawError()) ;
			return ;
		}
		
		// Crypt a TestCryptDir file to recipients with each public key
		
		beginOp("RSA Encrypting Directory " + sOutputDir) ;
		
		m_ccaSafeAsym.asymEncryptDir(sInputDir,
									  sOutputDir,
									  "Recipient_List");
		
		System.out.println("RSA Encrypting Directory : terminated " + sOutputDir) ;
		
		if(!m_ccaSafeAsym.isOperationOK())
		{
			System.out.println(m_ccaSafeAsym.getRawError()) ;
			return ;
		}
		
		return;
	}	
		
	/**
	 * the TestCryptDir method for the asymmetric decryption mecanisms of SafeAPI
	 * 
	 * @param	sInputDir	the encrypted directory
	 * @param	sFileDec	the resulting decrypted file
	 */
	
	public void testRSADecryptDir(String sInputDir, String  sOutputDir)
		throws Exception 
	{
		System.out.println() ;
		System.out.println("*** Testing RSA Directory Decryption ***") ;
		System.out.println() ;
		
		String sKeyId = new String() ;
		int nCnt = 1;
			
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;		
					
		beginOp("Decrypting Directory " + sInputDir) ;
		
		m_ccaSafeAsym.asymDecryptDir(sKeyId,
									  this.m_sPassphrase.toCharArray(),
									  sInputDir,
									  sOutputDir) ;
		
		stopOp(" decrypted Directory : " + sOutputDir) ;
		
		if(!m_ccaSafeAsym.isOperationOK())
		{
			System.out.println(m_ccaSafeAsym.getRawError()) ;
			return ;
		}
			
	}		
				
	// safeLogic debug tools
	
	public static void SetDebug(boolean bDebug)
	{
		CSA_DEBUG = bDebug;
	}	
	
	private static void DebugPrintln(String sIn)
	{
		if (CSA_DEBUG)
			System.out.println(sIn);
	}		
}
	
