// TestRSASign.java : SafeAPI Package.
// Class Description      : Auto TestCryptDir of  SafeAPI asymmetric sign/verify 
//					 
// 16/07/00 19:30 NDP- adapt from old CSaCryTestPKI
// 17/07/00 00:55 NDP- implement all sign/verify methods
// 17/07/00 19:45 NDP- final tests
// 16/09/00 17:50 NDP - new package name
// 06/10/00 12:35 NDP - use safeAPI v1.04 algorithm
// 01/03/01 15:35 NDP - use SafeAPI v1.05 new raw RSA methods

package com.safeapi.test;

import java.io.File;
import java.math.BigInteger;
import com.safeapi.CryptoAsym;
import com.safeapi.CryptoHash;
import com.safeapi.CryptoAsymRawRSA;
import com.safeapi.util.ParmsInternal;
import com.safelogic.crytools.ParmCry;
import com.safelogic.utilx.Hex;

/**
 * This Class is designed to TestCryptDir the public key interface of SafeAPI.
 * <br>
 * This includes all asymmetric encryption and signature methods.
 * 
 */

public class TestRSASign extends TestTimer
{
	// set to true to force debug
	public static boolean CSA_DEBUG = false;
	
	// attributes
	CryptoAsym m_ccaSafeTest ;
    CryptoHash m_hash;
    CryptoAsymRawRSA m_rawRsa;
    
	String m_sEmail;
	String m_sPassphrase ;	
	
	/**
	 * Constructor
	 * 
	 * @param sEmail		E mail used to builds the symmetric key
	 * @param sPasshrase	passphrase for key
	 */
	
	public TestRSASign(String sEmail,
							 String sPasshrase)
	{
		// Create a SafeAPI instance
		super() ;
		m_ccaSafeTest = new CryptoAsym() ;
		m_hash = new CryptoHash();
        m_rawRsa = new CryptoAsymRawRSA();
        
		this.m_sEmail		= sEmail;
		this.m_sPassphrase	= sPasshrase;		
	}

	// methods
	
	
	/**
	 * Raw Sign and verify a buffer with SignBuffer()/VerifyBuffer()
	 */
	
	public void testRawSignBuffer(String sBuffer)
		throws Exception
	{
		byte[] bSignature ;
		
		System.out.println() ;
		System.out.println("*** Testing Raw Buffer Signature ***") ;
		System.out.println() ;
		
		beginOp("Raw Signing Buffer " + sBuffer) ;
		
		int nCnt = 1;
		String sKeyId = new String() ;
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;			
		
        /*
		RawRSAPrivateKey privKey = 
			(RawRSAPrivateKey) m_rawRsa.getPrivKey(sKeyId, 
                                                        m_sPassphrase.toCharArray());
		
		BigInteger biD_Exponent1 = privKey.getExponent();
		BigInteger biP_Factor	 = privKey.getP();
		BigInteger biQ_Factor	 = privKey.getQ();	
		*/
        
        m_rawRsa.loadPrivateKey(sKeyId, m_sPassphrase.toCharArray());
        
        BigInteger biD_Exponent1 = new BigInteger(m_rawRsa.getPrivateKeyExponent());
        BigInteger biP_Factor    = new BigInteger(m_rawRsa.getP());
        BigInteger biQ_Factor    = new BigInteger(m_rawRsa.getQ());  
                
		byte [] bD_Exponent1 = m_rawRsa.getPrivateKeyExponent();
		byte [] bP_Factor	 = m_rawRsa.getP();
		byte [] bQ_Factor	 = m_rawRsa.getQ();		
				
		bSignature = m_rawRsa.signBufferRawKey(bD_Exponent1, bP_Factor, bQ_Factor, 
											  sBuffer.getBytes()) ;
	
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			throw new Exception("Failed");
		}
		else
		{
			stopOp("Raw sign : OK ") ;
		}
				
		boolean bVerify = false ;
		
		// change signature value : use this to check "signature disapproved"
		//bSignature[0] += 1 ;
		
		beginOp("Raw Verifying Buffer " + sBuffer) ;
		
		//RawRSAPublicKey pubKey = (RawRSAPublicKey) m_rawRsa.getPubKey(sKeyId);		
        m_rawRsa.loadPublicKey(sKeyId);
        
        /*
		BigInteger biN_Modulus  = new BigInteger(m_rawRsa.getModulus());
		BigInteger biD_Exponent2 = new BigInteger(m_rawRsa.getPublicKeyExponent());		
		
		byte [] bN_Modulus  = biN_Modulus.toByteArray();
		byte [] bD_Exponent2 = biD_Exponent2.toByteArray();		
		*/
        
        byte [] bN_Modulus  = m_rawRsa.getModulus();
        byte [] bD_Exponent2 = m_rawRsa.getPublicKeyExponent();         
        
		bVerify = m_rawRsa.verifyBufferRawKey(bN_Modulus, bD_Exponent2,
											 sBuffer.getBytes(),									
											 bSignature) ;
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
		else
		{
			stopOp("Raw verify : OK") ;
		}
				
		if(bVerify)
		{
			System.out.println("Raw Buffer Signature approved.") ;
		}
		else
		{
			throw new Exception("Buffer Signature disapproved.") ;
		}
	}		
	
	
	/**
	 * Sign and verify a buffer with SignBuffer()/VerifyBuffer()
	 */
	
	public void testSignBuffer(String sBuffer)
		throws Exception
	{
		byte[] bSignature ;
		
		System.out.println() ;
		System.out.println("*** Testing Buffer Signature ***") ;
		System.out.println() ;
		
		beginOp("Signing Buffer " + sBuffer) ;
		
		int nCnt = 1;
		String sKeyId = new String() ;
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;			
		
		bSignature = m_ccaSafeTest.signBuffer(sKeyId,
											   this.m_sPassphrase.toCharArray(),
											   sBuffer.getBytes()) ;
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
		else
		{
			stopOp("sign : OK ") ;
		}
				
		boolean bVerify = false ;
		
		// change signature value : use this to check "signature disapproved"
		//bSignature[0] += 1 ;
		
		beginOp("Verifying Buffer " + sBuffer) ;
		
		bVerify = m_ccaSafeTest.verifyBuffer(sKeyId,
											 sBuffer.getBytes(),									
											 bSignature) ;
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
		else
		{
			stopOp("verify : OK") ;
		}
				
		if(bVerify)
		{
			System.out.println("Buffer Signature approved.") ;
		}
		else
		{
			throw new Exception("Buffer Signature disapproved.") ;
		}
	}	
	
	
	/**
	 * Sign and verify a file with RawSignFile()/RawVerifyFile()
	 * @param sFile		File to sign/verify
	 */
	
	public void testRawSignFile(String sFile)
		throws Exception
	{
		byte[] bSignature ;
		
		System.out.println() ;
		System.out.println("*** Testing Raw File Signature ***") ;
		System.out.println() ;
		
		beginOp("Signing file " + sFile) ;
		
		int nCnt = 1;
		String sKeyId = new String() ;
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;			
		
		bSignature = m_ccaSafeTest.rawSignFile(sKeyId,
											   this.m_sPassphrase.toCharArray(),
											   sFile) ;
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
		else
		{
			stopOp("sign : OK ") ;
		}
				
		boolean bVerify = false ;
		
		// change signature value : use this to check "signature disapproved"
		//bSignature[0] += 1 ;
		
		beginOp("Verifying file " + sFile) ;
		
		bVerify = m_ccaSafeTest.rawVerifyFile(sKeyId,
											 sFile,										
											 bSignature) ;
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
		else
		{
			stopOp("verify : OK") ;
		}
				
		if(bVerify)
		{
			System.out.println("Signature approved.") ;
		}
		else
		{
			throw new Exception("Signature disapproved.") ;
		}
	}
	
	/**
	 * Sign and verify a file with SignFile()/VerifyFile()
	 * @param	sFile		File to sign/verify
	 * @param	sFileSign	File signature
	 */
	
	public void testSignFile(String sFile, String sFileSign)
		throws Exception
	{
		//byte[] bSignature ;
		
		System.out.println() ;
		System.out.println("*** Testing File Signature ***") ;
		System.out.println() ;
		
		beginOp("Signing file " + sFile) ;
		
		int nCnt = 1;
		String sKeyId = new String() ;
		sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
				 + nCnt + ParmsInternal.KEY_NAME_SEPARATOR 
				 + ParmCry.CSA_ALGO_RSA ;			
		
		m_ccaSafeTest.signFile(sKeyId,
								  this.m_sPassphrase.toCharArray(),
								  sFile,
								  sFileSign) ;
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
		else
		{
			stopOp("sign : OK ") ;
		}
				
		boolean bVerify = false ;
		
		// change signature value : use this to check "signature disapproved"
		//bSignature[0] += 1 ;
		
		beginOp("Verifying file " + sFile) ;
		
		bVerify = m_ccaSafeTest.verifyFile(sKeyId,
											 sFile,	
											 sFileSign) ;
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
		else
		{
			stopOp("verify : OK") ;
		}
				
		if(bVerify)
		{
			System.out.println("Signature approved.") ;
		}
		else
		{
			throw new Exception("Signature disapproved.") ;
		}
	}	
	
	
	/**
	 * TestCryptDir the encryption/signature mecanism of SafeAPI.
	 * <BR> NOTE : 
	 * in this case, the signature is written to the encrypted file.
	 * 
	 * @param	sFileDec	the file to encrypt & sign
	 * @param	sFileEnc	the  resulting encrypted a signed file
	 */
	
	public void testEncryptSign(String sFileDec, String sFileEnc)
	{
		//String sKeyId		= "moi@domain.web_2_RSA" ;
		//String sPass		= "ma passphrase2" ;
		
		//String sSignKeyId	= "moi@domain.web_1_RSA" ;
		//String sPassSign	= "ma passphrase1" ;
		
		String sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
						+ 2 + ParmsInternal.KEY_NAME_SEPARATOR 
						+ ParmCry.CSA_ALGO_RSA ;	
		
		String sSignKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
						+ 1 + ParmsInternal.KEY_NAME_SEPARATOR 
						+ ParmCry.CSA_ALGO_RSA ;	
		
		System.out.println() ;
		System.out.println("*** Testing Encrypt & Sign on file***") ;
		System.out.println() ;		
		
		m_ccaSafeTest.createRecipients("Recipient_List") ;
		m_ccaSafeTest.addRecipient("Recipient_List", sKeyId) ;
		
		// Crypt a TestCryptDir file to recipients with each public key
		
		beginOp("Signing and encrypting file " + sFileDec) ;
		
		m_ccaSafeTest.encryptAndSign(sSignKeyId,
									 this.m_sPassphrase.toCharArray(),
									 "Recipient_List",
									 sFileDec, 
									 sFileEnc);
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}
		
		stopOp("encrypted file : " + sFileEnc) ;
		
	}
	
	/**
	 * TestCryptDir the decryption/verification mecanism of SafeAPI.
	 * <BR> NOTE : 
	 * in this case, the signature is written to the encrypted file.
	 * 
	 * @param	sFileEnc	the  encrypted and signed file
	 * @param	sFileDec	the decrypted & verified file

	 */
	
	public void testDecryptVerify(String sFileEnc, String sFileDec)
		throws Exception
	{
		//String sKeyId		= "moi@domain.web_2_RSA" ;
		//String sPass		= "ma passphrase2" ;
		
		//String sSignKeyId	= "moi@domain.web_1_RSA" ;
		//String sPassSign	= "ma passphrase1" ;
		
		String sKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
						+ 2 + ParmsInternal.KEY_NAME_SEPARATOR 
						+ ParmCry.CSA_ALGO_RSA ;	
		
		String sSignKeyId = this.m_sEmail + ParmsInternal.KEY_NAME_SEPARATOR 
						+ 1 + ParmsInternal.KEY_NAME_SEPARATOR 
						+ ParmCry.CSA_ALGO_RSA ;	
		
		System.out.println() ;
		System.out.println("*** Testing Decrypt & Verify on file***") ;
		System.out.println() ;		
			
		int nMyFlag ;
		
		beginOp("Decrypting and verifying file " + sFileEnc) ;
		
		nMyFlag = m_ccaSafeTest.decryptAndVerify(sKeyId,
												 this.m_sPassphrase.toCharArray(),
												 sSignKeyId,
												 sFileEnc,
												 sFileDec) ;
		
		if(!m_ccaSafeTest.isOperationOK())
		{
			System.out.println(m_ccaSafeTest.getRawError()) ;
			return ;
		}		

		stopOp("decrypted file: " + sFileDec) ;
		
		System.out.print("Signature : ") ;
		if(nMyFlag > 0)
		{
			System.out.println("approved") ;
		}
		
		if(nMyFlag < 0)
		{
			System.out.println("disapproved") ;
			throw new Exception("disapproved");
		}
		
		if(nMyFlag == 0)
		{
			System.out.println("not signed") ;
			throw new Exception("not signed");
		}
	}	
	
	/**
	 * Test symmetric methods: File encryption/decryption
	 * @param	sFile	File to use for encryption/decryption
	 */
	
	public void testRSASignCryptDecryptFile(String sFileDec)
		throws Exception
	{				
		String sFileEnc = sFileDec + ".rsa_sign_enc";
		String sFileDec2= sFileDec + ".rsa_sign_dec";
		new File(sFileEnc).delete();
		new File(sFileDec2).delete();
		
		this.testEncryptSign(sFileDec, sFileEnc);
		this.testDecryptVerify(sFileEnc, sFileDec2);
		this.testCompFile(sFileDec, sFileDec2);
		
	}			
	
	/**
	 * MD5 compare of two files
	 * @param sFile1	File to compare
	 * @param sFile2	File to MD5 compare with File1
	 */
	
	public void testCompFile(String sFile1 , String sFile2)
		throws Exception
	{		
		String sHash1 = Hex.toString( this.m_hash.hashDataFile("MD5", sFile1));
		String sHash2 = Hex.toString( this.m_hash.hashDataFile("MD5", sFile1));
		
		if (sHash1.equals(sHash2))
		{
			System.out.println("MD5 comp: " + sFile1 + "=" + sFile2);
		}
		else {
			throw new Exception("MD5 comp: " + sFile1 + "!=" + sFile2);
		}
	}		
	
	// safeLogic debug tools
	
	public static void SetDebug(boolean bDebug)
	{
		CSA_DEBUG = bDebug;
	}	
	
	private static void DebugPrintln(String sIn)
	{
		if (CSA_DEBUG)
			System.out.println(sIn);
	}		
}
	
