// TestSym.java : SafeAPI Package.
// Class Description  : Auto TestCryptDir of  SafeAPI symmetric crypto
//					 
// 22/11/99 15:30 GR - 1st Release
// 15/07/00 19:55 NDP - New methods & throw fatal exception
// 16/07/00 16:05 NDP - comments & new names
// 16/09/00 18:15 NDP - new package name & throws new eception
// 01/03/01 16:10 NDP - use GetSecretKey
// 13/09/01 12:25 NDP - set back CSA_DEBUG to false afer 1.05b debug

package com.safeapi.test;

import java.io.File;
import java.util.Vector;
import com.safeapi.CryptoHash;
import com.safeapi.CryptoSym;
import com.safelogic.utilx.Hex;

public class TestSym extends TestTimer
{
	// set to true to force debug
	public static boolean CSA_DEBUG = false;

	// Default Vector
	public static String DEFAULT_VECTOR = "FEDCBA9876543210";

	// attributes
	CryptoSym m_safeapi;
    CryptoHash m_hash;
    
	String m_sAlgo;
	String m_sEmail;
	String m_sPassphrase;

	// Key ID built from e mail
	String m_sKeyId;

	// List for symmetric crypto references
	Vector m_vSymTestList;

	/**
	 * Constructor
	 * <br>
	 * @param algorithm		symmetric algorithm to use
	 * @param sEmail		E mail used to builds the symmetric key
	 * @param sPasshrase	passphrase for key
	 */

	public TestSym(String sAlgorithm, String sEmail, String sPasshrase)
	{
		super();
		m_safeapi = new CryptoSym();
        m_hash = new CryptoHash();
        
		m_sAlgo = sAlgorithm;
		m_sEmail = sEmail;
		m_sPassphrase = sPasshrase;

	}

	// methods

	/**
	 * generate a secret key
	 */
	public void testGenKey()
        throws Exception
	{
		int nIndex = 1;

		byte[] bSeed = new byte[16];

		byte[] bSeed192 = m_safeapi.getRandomBytes(this.m_sPassphrase);
		DebugPrintln("bSeed192.length :" + bSeed192.length);

		System.out.println();
		System.out.println("*** Testing Symmetric Key Generation ***");
		System.out.println();

		// Strong seedind
		for (int nCnt = 0; nCnt < bSeed.length; nCnt++)
		{
			//DebugPrintln("nCnt :" + nCnt);
			bSeed[nCnt] = bSeed192[nCnt];
		}

		beginOp("Generating Key for algorithm " + m_sAlgo);

		this.m_safeapi.genSecretKey(
			m_sAlgo,
			m_sPassphrase.toCharArray(),
			128,
			bSeed,
			m_sEmail,
			nIndex);

		if (m_safeapi.isOperationOK())
		{
			m_sKeyId = m_sEmail + "_" + nIndex + "_" + m_sAlgo;
			stopOp("Key " + m_sKeyId + " generated");
		}
		else
		{
			System.out.println("Error : " + m_safeapi.getRawError());
            throw new Exception();
		}
	}

	/**
	 * raw encrypt the specified String
	 * 
	 * @param	sDec String to encrypt
	 * @return	the  resulting encrypted String
	 */

	public String testRawSymEncrypt(String sDec) throws Exception
	{
		System.out.println();
		System.out.println("*** Testing Raw Symmetric Buffer Encryption ***");
		System.out.println();

		beginOp(m_sAlgo + " encrypting : " + sDec);

		String sHexaKey =
			m_safeapi.getSecretKey(
				this.m_sAlgo,
				this.m_sPassphrase.toCharArray(),
				this.m_sEmail,
				1);

		System.out.println("sHexaKey :" + sHexaKey + ":");

		//byte []bEnc = m_safeapi.EncryptBuffer(m_sKeyId, m_sPassphrase,  sDec.getBytes());

		byte[] bEnc =
			m_safeapi.encryptBufferRawKey(
				this.m_sAlgo,
				Hex.fromString(sHexaKey),
				Hex.fromString(this.DEFAULT_VECTOR),
				sDec.getBytes());

		if (m_safeapi.isOperationOK())
		{
			stopOp("Raw encrypted to : " + Hex.toString(bEnc) + ":");
		}
		else
		{
			System.out.println("Error : 1"); // + m_safeapi.getRawError()) ;
			throw new Exception("Failed: " + sDec + " encryption/decryption");
		}

		return new String(Hex.toString(bEnc));
	}

	/**
	 * decrypt the specified String
	 * 
	 * @param	sEnc String to encrypt
	 * @return	the  resulting encrypted String
	 */

	public String testRawSymDecrypt(String sEnc) throws Exception
	{
		System.out.println();
		System.out.println("*** Testing Raw Symmetric Buffer Decryption ***");
		System.out.println();

		beginOp(m_sAlgo + " decrypting : " + sEnc);

		String sHexaKey =
			m_safeapi.getSecretKey(
				this.m_sAlgo,
				this.m_sPassphrase.toCharArray(),
				this.m_sEmail,
				1);

		System.out.println("sHexaKey :" + sHexaKey + ":");

		//byte []bDec = m_safeapi.DecryptBuffer(m_sKeyId, m_sPassphrase, 
		//									  Hex.fromString(sEnc));

		byte[] bDec =
			m_safeapi.decryptBufferRawKey(
				this.m_sAlgo,
				Hex.fromString(sHexaKey),
				Hex.fromString(this.DEFAULT_VECTOR),
				Hex.fromString(sEnc));

		if (m_safeapi.isOperationOK())
		{
			stopOp("raw decrypted to : " + new String(bDec) + ":");
		}
		else
		{
			System.out.println("Error : 1"); // + m_safeapi.getRawError()) ;
			throw new Exception("Failed: " + sEnc + " encryption/decryption");
		}

		return new String(bDec);
	}

	/**
	 * Test symmetric methods: buffer encryption/decryption
	 * @param	sDec	Buffer to use for encryption/decryption
	 */

	public void testRawSymCryptDecryptBuffer(String sDec) throws Exception
	{
		String sEnc = new String();
		String sDec2 = new String();

		sEnc = this.testRawSymEncrypt(sDec);
		sDec2 = this.testRawSymDecrypt(sEnc);

		this.testCompString(sDec, sDec2);
	}

	/**
	 * encrypt the specified String
	 * 
	 * @param	sDec String to encrypt
	 * @return	the  resulting encrypted String
	 */

	public String testSymEncrypt(String sDec) throws Exception
	{
		System.out.println();
		System.out.println("*** Testing Symmetric Buffer Encryption ***");
		System.out.println();

		beginOp(m_sAlgo + " encrypting : " + sDec);

		byte[] bEnc =
			m_safeapi.encryptBuffer(
				m_sKeyId,
				m_sPassphrase.toCharArray(),
				sDec.getBytes());

		if (m_safeapi.isOperationOK())
		{
			stopOp("encrypted to : " + Hex.toString(bEnc));
		}
		else
		{
			System.out.println("Error : 1 " + m_safeapi.getRawError()) ;
            System.out.println("Error : 1 " + m_safeapi.getRegisteredError()) ;
			throw new Exception("Failed: " + sDec + " encryption/decryption");
		}

		return new String(Hex.toString(bEnc));
	}

	/**
	 * decrypt the specified String
	 * 
	 * @param	sEnc String to encrypt
	 * @return	the  resulting encrypted String
	 */

	public String testSymDecrypt(String sEnc) throws Exception
	{
		System.out.println();
		System.out.println("*** Testing Symmetric Buffer Decryption ***");
		System.out.println();

		beginOp(m_sAlgo + " decrypting : " + sEnc);

		byte[] bDec =
			m_safeapi.decryptBuffer(
				m_sKeyId,
				m_sPassphrase.toCharArray(),
				Hex.fromString(sEnc));

		if (m_safeapi.isOperationOK())
		{
			stopOp("decrypted to : " + new String(bDec));
		}
		else
		{
			System.out.println("Error : 1"); // + m_safeapi.getRawError()) ;
			throw new Exception("Failed: " + sEnc + " encryption/decryption");
		}

		return new String(bDec);
	}

	/**
	 * encrypt the specified file
	 * 
	 * @param	sFileDec	the file to encrypt
	 * @param	sFileEnc	the  resulting encrypted file
	 */

	public void testSymEncrypt(String sFileDec, String sFileEnc)
		throws Exception
	{

		System.out.println();
		System.out.println("*** Testing Symmetric File Encryption ***");
		System.out.println();

		beginOp(m_sAlgo + " encrypting : " + sFileDec);

		m_safeapi.encryptFile(
			m_sKeyId,
			m_sPassphrase.toCharArray(),
			sFileDec,
			sFileEnc);

		if (m_safeapi.isOperationOK())
		{
			stopOp("encrypted to : " + sFileEnc);
		}
		else
		{
			System.out.println("Error : 1"); // + m_safeapi.getRawError()) ;
			throw new Exception("Failed: encrypting : " + sFileDec);
		}

	}

	/**
	 * decrypt the specified file
	 * 
	 * @param	sFileEnc	the encrypted file 
	 * @param	sFileDec	the resulting decrypted file
	 */

	public void testSymDecrypt(String sFileEnc, String sFileDec)
		throws Exception
	{
		System.out.println();
		System.out.println("*** Testing Symmetric File Decryption ***");
		System.out.println();

		beginOp(m_sAlgo + " decrypting : " + sFileDec + ".enc");

		m_safeapi.decryptFile(m_sKeyId, 
                              m_sPassphrase.toCharArray(), 
                              sFileEnc, sFileDec);

		if (m_safeapi.isOperationOK())
		{
			stopOp("decrypted to : " + sFileDec);
		}
		else
		{
			System.out.println("Error : " + m_safeapi.getRawError());
			throw new Exception("Failed: decrypting : " + sFileDec);
		}
	}

	/**
	 * comparaison of two Strings
	 * @param s1	String to compare
	 * @param s2	String  compare with s2
	 */

	public void testCompString(String s1, String s2) throws Exception
	{
		if (s1.equals(s2))
		{
			System.out.println("comp: " + s1 + "=" + s2);
		}
		else
		{
			throw new Exception("comp: " + s1 + "!=" + s2);
		}
	}

	/**
	 * MD5 compare of two files
	 * @param sFile1	File to compare
	 * @param sFile2	File to MD5 compare with File1
	 */

	public void testCompFile(String sFile1, String sFile2) throws Exception
	{
		String sHash1 =
			Hex.toString(this.m_hash.hashDataFile("MD5", sFile1));
		String sHash2 =
			Hex.toString(this.m_hash.hashDataFile("MD5", sFile2));

		if (sHash1.equals(sHash2))
		{
			System.out.println("MD5 comp: " + sFile1 + "=" + sFile2);
		}
		else
		{
			throw new Exception("MD5 comp: " + sFile1 + "!=" + sFile2);
		}
	}

	/**
	 * Test symmetric methods: File encryption/decryption
	 * @param	sFile	File to use for encryption/decryption
	 */

	public void testSymCryptDecryptFile(String sFileDec) throws Exception
	{
		String sFileEnc = sFileDec + ".enc";
		String sFileDec2 = sFileDec + ".dec";
		new File(sFileEnc).delete();
		new File(sFileDec2).delete();

		this.testSymEncrypt(sFileDec, sFileEnc);
		this.testSymDecrypt(sFileEnc, sFileDec2);
		this.testCompFile(sFileDec, sFileDec2);

	}

	/**
	 * Test symmetric methods: buffer encryption/decryption
	 * @param	sDec	Buffer to use for encryption/decryption
	 */

	public void testSymCryptDecryptBuffer(String sDec) throws Exception
	{
		String sEnc = new String();
		String sDec2 = new String();

		sEnc = this.testSymEncrypt(sDec);
		sDec2 = this.testSymDecrypt(sEnc);

		this.testCompString(sDec, sDec2);
	}

	/**
	 * create the reference TestCryptDir lists (hash and symmetric encryption)
	 */

	public void createList()
	{

		m_vSymTestList = new Vector();

		m_vSymTestList.addElement(
			new TestSymRef(
				"Blowfish",
				"0123456789ABCDEFF0E1D2C3B4A59687",
				"FEDCBA9876543210",
				"37363534333231204E6F77206973207468652074696D6520666F722000",
				"E73214A2822139CAF26ECF6D2EB9E76E3DA3DE04D1517200519D57A6C3"));

		m_vSymTestList.addElement(
			new TestSymRef(
				"IDEA",
				"729A27ED8F5C3E8BAF16560D14C90B43",
				"C121A1B050D8286C",
				"D53FABBF94FF8B5F",
				"E42323CAF932B933"));

		m_vSymTestList.addElement(
			new TestSymRef(
				"IDEA",
				"729A27ED8F5C3E8BAF16560D14C90B43",
				"C121A1B050D8286C",
				"94FF8B5F",
				"A5E3032A"));
	}

	/**
	 * compute the reference TestCryptDir for SafeAPI symmetric encryption algorithms
	 * with Raw buffers
	 */

	public void testSymRaw() throws Exception
	{
		// Create the list
		TestSymRef srCurrent;
		createList();

		System.out.println();
		System.out.println("          #######################");
		System.out.println("          # Symmetric Encryption #");
		System.out.println("          #######################");
		System.out.println();

		for (int nCnt = 0; nCnt < m_vSymTestList.size(); nCnt++)
		{
			srCurrent = (TestSymRef) m_vSymTestList.elementAt(nCnt);

			System.out.println(
				"***********************************************");
			System.out.println("Test Reference for " + srCurrent.getAlgo());
			System.out.println();

			System.out.println("Input : ");
			System.out.println("Buffer    : " + srCurrent.getRefIn());
			System.out.println("Key       : " + srCurrent.getKey());
			System.out.println("IV        : " + srCurrent.getIV());
			System.out.println();

			System.out.println("Output : ");

			System.out.print("Reference : ");
			System.out.println(srCurrent.getRefOut().toUpperCase());

			System.out.print("SafeAPI   : ");

			byte[] bRefIn;
			byte[] bRefKey;
			byte[] bRefIV;
			byte[] bSafeOut;

			bRefIn = Hex.fromString(srCurrent.getRefIn());
			bRefKey = Hex.fromString(srCurrent.getKey());

			bRefIV = Hex.fromString(srCurrent.getIV());

			bSafeOut =
				m_safeapi.encryptBufferRawKey(
					srCurrent.getAlgo(),
					bRefKey,
					bRefIV,
					bRefIn);

			if (!m_safeapi.isOperationOK())
			{
				System.out.println("Error : ");
				System.out.println(m_safeapi.getRawError());
				return;
			}

			String sCompute = Hex.toString(bSafeOut);

			System.out.print(sCompute);
			System.out.println();

			if (!srCurrent.getRefOut().equalsIgnoreCase(sCompute))
			{
				throw new Exception(" ***FAILED " + srCurrent.getAlgo());
			}
		}

		System.out.println("***********************************************");
	}

	// safeLogic debug tools

	public static void SetDebug(boolean bDebug)
	{
		CSA_DEBUG = bDebug;
	}

	private static void DebugPrintln(String sIn)
	{
		if (CSA_DEBUG)
			System.out.println(sIn);
	}

}

// End