// all.java : SafeAPI Package.
// Class Description   : Auto TestCryptDir of all SafeAPI
//						 
// 15/07/00 13:10 NDP - Creation
// 16/07/00 17:35 NDP - multi algo pass & add RSA tests
// 16/07/00 00:30 NDP - add sign tests
// 17/07/00 19:35 NDP - final tests
// 15/09/00 20:00 NDP - +s and +k are two parameters
// 16/09/00 17:40 NDP - works with dynamic files & with unix
// 16/09/00 18:05 NDP - new package name
// 01/10/00 20:20 NDP - TestCryptDir rsa sign/verify before file encryption
// 03/10/00 20:40 NDP - TestCryptDir raw rsa sign/verify on files before file encryption
// 04/10/00 13:25 NDP - TestCryptDir Sign file detached on files before file encryption
// 04/10/00 20:20 NDP - comments
// 06/10/00 12:35 NDP - use safeAPI v1.04 algorithm
// 01/03/01 18:45 NDP - use new Raw key RSA encrypt/decrypt & sign/verify & CallAsymCryptFile
// 15/12/01 14:05 NDP - add TestConv calls
// 04/06/02 19:45 NDP - use common.jar for file copy
// 13/06/02 18:50 NDP - add TestRSA
// 09/01/03 19:45 NDP - use utilx only
// 10/02/03 18:50 NDP - use utilx.jar instead of common Jar
// 11/05/04 18:05 NDP - comments

package com.safeapi.test;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import com.safeapi.CryptoSym;
import com.safelogic.crytools.ParmCry;
import com.safelogic.utilx.dir.Directory;
import com.safelogic.utilx.io.file.IniFileManager;

public class all
{
    
	public static String USER_DIR = System.getProperty("user.dir");
	
	public static String KEY_EMAIL = "test_safeapi@safelogic.com";
	public static String KEY_PASS  = "test_safeapi@safelogic.com";
	
	public static String DEFAULT_FILE	= "safeapi_test.zip";
	public static int MAX_ROUND			= 12;
	
    //
    // Socket buffer size
    //

    public final static int CSA_SOCKET_BUFFER = 2048;    
    
	private static void DisplayUsage()
	{
		System.out.println("java com.safeapi.all <option switches>");
		System.out.println("where:");		
		System.out.println("- option switches: <-s|+s> <-k|+k>");
		System.out.println("  +s|-s : seed generation | no seed generation");
		System.out.println("  +k|-k : key generation  | no key generation");		
	}	
		
    /**
     * Create specified file using a safeapi file from classpath
     * @param sInFile   full file name to duplicate
     * @param sOutFile  full file name of target file   
     *  
     */
    
    public static void CopyFile(String sInFile, String sOutFile)
        throws IOException
    {
        //int     byteOrMinus1;
        
        BufferedInputStream  bin = 
            new BufferedInputStream( new FileInputStream(sInFile) );
        
        BufferedOutputStream bos = 
            new BufferedOutputStream(new FileOutputStream(sOutFile)) ;    
        
        int  nBytesRead;
        byte bArray[]    = new byte[CSA_SOCKET_BUFFER];
        
        while ((nBytesRead = bin.read(bArray)) > 0)
        {     
            bos.write(bArray, 0, nBytesRead);
        }
        
        bin.close();
        bos.close();
    }           
        
	
	/**
	 * TestCryptDir buffer & file encrytpion 
	 * @param sFile			File to use for tests
	 * @param sOptionKey	Batch option
	 * @param sAlgo			symmetric algorithm
	 * @param bRaw			if true ==> use Raw symmetric buffer encryption
	 */
	
	private static void AutoTestSym(String sFile, 
									String sOptionKey, 
									String sAlgo,
									boolean bRaw)
		throws Exception
	{
		String sBuffer	= "hello world";
		
		// Generate symmetric keys
		TestSym csa_test_sym = new TestSym(sAlgo, KEY_EMAIL, KEY_PASS);
		
		if (sOptionKey.indexOf("+k") != -1)
		{		
			csa_test_sym.testGenKey();	
		}
		
		// 1) Test raw datas provided by inventors
		csa_test_sym.testSymRaw();					
		
		// 2.1) Test buffer encryption
		for (int i= 0; i < MAX_ROUND; i++)
		{
			sBuffer+= new Integer(i).toString();
			csa_test_sym.testSymCryptDecryptBuffer(sBuffer);
		}
		
		if (bRaw)
		{
			// 2.2) Test buffer encryption with RAW secret key
			sBuffer	= "hello world";
			
			for (int i= 0; i < MAX_ROUND; i++)
			{
				sBuffer+= new Integer(i).toString();
				csa_test_sym.testRawSymCryptDecryptBuffer(sBuffer);
			}		
		}
		
		// Test File crypto on 1 file
		csa_test_sym.testSymCryptDecryptFile(sFile);		
	}
	
	/**
	 * M A I N
	 */
	public static void main(String[] args)
		throws Exception 
	{        
	    	    
		if (args.length != 2)
		{
			DisplayUsage();
			return;
		}
		
        //SafeProvider.add();
        
		String sOptionSeed = new String(args[0]);
		String sOptionKey  = new String(args[1]);
		
		// Build a seed file
		
		CryptoSym safeapi	= new CryptoSym();
		
		if (sOptionSeed.indexOf("+s") != -1)
		{
			safeapi.createSeedFile("gnration de seed",
							   "Veuillez taper lentement sur le clavier");
		}
				
		// Create a duplicate image of a safeapi file
		IniFileManager iniFileManager = new IniFileManager();
        
        String sFileIn  = iniFileManager.getIniFilename("cryptix.jar");
		String sFileOut	= USER_DIR;
		
		sFileOut= Directory.AddSep(sFileOut);
		sFileOut+=DEFAULT_FILE;
        System.out.println("File in: " + sFileIn);
		System.out.println("Creation of working file: " + sFileOut);
		CopyFile(sFileIn, sFileOut);		
		
		
		// Test Converters
		TestConv csa_test_conv = new TestConv();
		csa_test_conv.testConv("abc");
		csa_test_conv.testConv("safeapi");
		csa_test_conv.testConv("hello world");
		
		// Test Hash References values
		TestHashBase csa_test_hash_base = new TestHashBase();
		csa_test_hash_base.testHashBuffers();
		csa_test_hash_base.testHashFiles();
		
		// Test symmetric encryption/decryption for 3 algos			
		AutoTestSym(sFileOut, sOptionKey, ParmCry.CSA_ALGO_BLOWFISH, true);
		AutoTestSym(sFileOut, sOptionKey, ParmCry.CSA_ALGO_CAST_128, true);
		AutoTestSym(sFileOut, sOptionKey, ParmCry.CSA_ALGO_IDEA, true);
				
		
		// Test RSA Encryption/decryption
		
		// Generate asymmetric keys
		TestRSACrypt csa_test_rsa = new TestRSACrypt(KEY_EMAIL, KEY_PASS);
		
		if (sOptionKey.indexOf("+k") != -1)
		{		
			csa_test_rsa.testKeyPairGen(1024);
		}
		
		// Test File crypto on 1 file
        // HACK csa_test_rsa.testRSACryptDecryptFile(sFileOut);  
		csa_test_rsa.testRSACryptDecryptFile("c:\\temp\\a.txt");			

				
		// Test buffer encryption
		String sBuffer = new TestHashBase().getHexaHash("MD5", KEY_PASS);
		
		// 2.1) Test buffer encryption
		for (int i= 0; i < MAX_ROUND; i++)
		{
			sBuffer+= new Integer(i).toString();
			csa_test_rsa.testRSACryptDecryptBuffer(sBuffer);
		}
		
		// 2.2) Test buffer encryption with RAW format key
		sBuffer = new TestHashBase().getHexaHash("MD5", KEY_PASS);
		for (int i= 0; i < MAX_ROUND; i++)
		{
			sBuffer+= new Integer(i).toString();
			csa_test_rsa.testRawRSACryptDecryptBuffer(sBuffer);
		}		
	
		// Generate asymmetric keys
		TestRSA rsaCrypt = new TestRSA(KEY_EMAIL, KEY_PASS);
		
		String sText =	TestRSA.STRING1 
						+ TestRSA.STRING2 
						+ TestRSA.STRING3;
		
		String sEnc = rsaCrypt.testRSAEncrypt(sText);		
		String sDec = rsaCrypt.testRSADecrypt(sEnc);		
		
		rsaCrypt.testCompString(sText, sDec);
		
		
		// Test RSA signature
		TestRSASign csa_test_sign = new TestRSASign(KEY_EMAIL, KEY_PASS);
		
		sBuffer = new TestHashBase().getHexaHash("MD5", KEY_PASS);
		for (int i= 0; i < MAX_ROUND; i++)
		{
			sBuffer+= new Integer(i).toString();
			csa_test_sign.testSignBuffer(sBuffer);
		}
		
		// Test RSA signature with Raw RSA key
		sBuffer = new TestHashBase().getHexaHash("MD5", KEY_PASS);
		for (int i= 0; i < MAX_ROUND; i++)
		{
			sBuffer+= new Integer(i).toString();
			csa_test_sign.testRawSignBuffer(sBuffer);
		}		
		
		
		// Sign file raw
		csa_test_sign.testRawSignFile(sFileOut);		
		
		// Sign file detached
		csa_test_sign.testSignFile(sFileOut, sFileOut + ".sig");		
		
	
		// Crypt+sign & decrypt+verify
		csa_test_sign.testRSASignCryptDecryptFile(sFileOut);
	}
			
}	