// CryptFile : SafeAPI Package.
// Class Description  : Example for SafeAPI Secure FTP usage
//					 
// 03/12/04 16:10 NDP - Creation

package com.safeapi.test.secureftp;

import com.safeapi.CryptoAsym;

/*
java com.safeapi.test.secureftp.CryptFile
*/

/**
 * This Class is and Example of SafeAPI Secure FTP usage
 * <br>
 * It allows to call the class from command level (DOS or Shell)  and encrypt/decrypt
 * a file with the test_secure_ftp@safelogic.com key Id.
 * 
 */

public class CryptFile
{
	
	/** A SafeAPI instance */
	public CryptoAsym m_safeapi ;
	
    // The RSA Key identifiers
    
    /** The Key_ID */
	public String m_sKey_ID;
    
    /** The passphrase of the private RSA Key */
	private String m_sPassphrase = "passphrase";
	
	
	/**
	 * Constructor
	 * 
	 * @param sEmail		E mail used to builds the symmetric key
	 * @param sPasshrase	passphrase for key
	 */
	
	public CryptFile(String sEmail)
	{
		// Create a SafeAPI instance
		m_safeapi = new CryptoAsym() ;
		
		this.m_sKey_ID		= sEmail + "_1_RSA"; 
	}

    /**
     * Displays the Licence Builder
     */
    public static void displayUsage()
    {    
        System.out.println(" ");
        System.out.println("Usage:");
        System.out.println("java com.safeapi.test.secureftp.CryptFile <-e|-d> <file in> <file out>");  
        System.out.println("where:");
        System.out.println("- -e|-d   : -e: encrypt input file | -d: decrypt input file");     
        System.out.println("- file in : file to encrypt/decrypt");  
        System.out.println("- file out: out file"); 
    }
    
	/**
	 * M A I N
	 */
	public static void main(String[] args)
		throws Exception 
	{
       
        if (args.length != 3)
        {
            displayUsage();
            return;
        }
        
        String sMode    = null;
        String sFileIn  = null;
        String sFileOut = null;
        
        sMode    = args[0];
        sFileIn  = args[1];
        sFileOut = args[2];
        
		CryptFile rsaCrypt = new CryptFile("test_secure_ftp@safelogic.com");
		
        if (sMode.equals("-e"))
        {
            rsaCrypt.encryptFile(sFileIn, sFileOut);
        }        
        else if (sMode.equals("-d"))
        {
            rsaCrypt.decryptFile(sFileIn, sFileOut);
        }        
        else
        {
            System.out.println("opration: -e or -d only. Nothing done."); 
        }
        
	}
	
    /**
     * The encryptFile method for the asymmetric decryption mecanisms of SafeAPI
     * 
     * @param   sFileIn  the file to encrypt 
     * @param   sFileOut the resulting encrypted file
     */
	
	public void encryptFile(String sFileIn , String sFileOut)
		throws Exception 
	{
		System.out.println() ;
		System.out.println("*** Testing RSA File Encryption ***") ;
		System.out.println() ;
		
		m_safeapi.encryptFile(this.m_sKey_ID, sFileIn, sFileOut );
	
        if (m_safeapi.isOperationOK())
        {
            System.out.println("OK. RSA file encryption is terminated for: " + sFileOut);
        }
        else
        {
            System.out.println(m_safeapi.getRawError());
        }
		
	}
	
		
	/**
	 * The decryptFile method for the asymmetric decryption mecanisms of SafeAPI
	 * 
	 * @param	sFileIn	 the encrypted file 
	 * @param	sFileOut the resulting decrypted file
	 */
	
	public void decryptFile(String sFileIn, String  sFileOut)
		throws Exception 
	{
		System.out.println() ;
		System.out.println("*** Testing RSA File Decryption ***") ;
		System.out.println() ;
				
		m_safeapi.decryptFile(m_sKey_ID,
									  this.m_sPassphrase.toCharArray(),
									  sFileIn,
									  sFileOut) ;
		
        if (m_safeapi.isOperationOK())
        {
            System.out.println("OK. RSA file encryption is terminated for: " + sFileOut);
        }
        else
        {
            System.out.println(m_safeapi.getRawError());
        }
			
	}
	
	
}
	
